@@ -21,7 +21,7 @@ static const char *EMPTY_STR = "";
 
 const char *MDAL_Version()
 {
-  return "0.7.4";
+  return "1.2.0";
 }
 
 MDAL_Status MDAL_LastStatus()
@@ -29,6 +29,27 @@ MDAL_Status MDAL_LastStatus()
   return MDAL::Log::getLastStatus();
 }
 
+MDAL_EXPORT void MDAL_ResetStatus()
+{
+  return MDAL::Log::resetLastStatus();
+}
+
+MDAL_EXPORT void MDAL_SetStatus( MDAL_LogLevel level, MDAL_Status status, const char *message )
+{
+  MDAL::Log::resetLastStatus();
+  switch ( level )
+  {
+    case MDAL_LogLevel::Error:
+      return MDAL::Log::error( status, message );
+    case MDAL_LogLevel::Warn:
+      return MDAL::Log::warning( status, message );
+    case MDAL_LogLevel::Info:
+      return MDAL::Log::info( message );
+    case MDAL_LogLevel::Debug:
+      return MDAL::Log::debug( message );
+  }
+}
+
 void MDAL_SetLoggerCallback( MDAL_LoggerCallback callback )
 {
   MDAL::Log::setLoggerCallback( callback );
@@ -150,6 +171,17 @@ const char *MDAL_DR_filters( MDAL_DriverH driver )
   return _return_str( d->filters() );
 }
 
+int MDAL_DR_faceVerticesMaximumCount( MDAL_DriverH driver )
+{
+  if ( !driver )
+  {
+    MDAL::Log::error( MDAL_Status::Err_MissingDriver, "Driver is not valid (null)" );
+    return -1;
+  }
+  MDAL::Driver *d = static_cast< MDAL::Driver * >( driver );
+  return d->faceVerticesMaximumCount();
+}
+
 ///////////////////////////////////////////////////////////////////////////////////////
 /// MESH
 ///////////////////////////////////////////////////////////////////////////////////////
@@ -194,6 +226,7 @@ const char *MDAL_MeshNames( const char *uri )
 
 void MDAL_SaveMesh( MDAL_MeshH mesh, const char *meshFile, const char *driver )
 {
+  MDAL::Log::resetLastStatus();
   if ( !meshFile )
   {
     MDAL::Log::error( MDAL_Status::Err_FileNotFound, "Mesh file is not valid (null)" );
@@ -222,10 +255,52 @@ void MDAL_SaveMesh( MDAL_MeshH mesh, const char *meshFile, const char *driver )
   }
 
   std::string filename( meshFile );
-  MDAL::DriverManager::instance().save( static_cast< MDAL::Mesh * >( mesh ), filename, driverName );
+
+  std::string uri = MDAL::buildMeshUri( filename, "", driverName );
+
+  MDAL::DriverManager::instance().save( static_cast< MDAL::Mesh * >( mesh ), uri );
 }
 
 
+void MDAL_SaveMeshWithUri( MDAL_MeshH mesh, const char *uri )
+{
+  MDAL::Log::resetLastStatus();
+
+  std::string meshFile;
+  std::string driverName;
+  std::string meshName;
+
+  MDAL::parseDriverAndMeshFromUri( uri, driverName, meshFile, meshName );
+
+  if ( meshFile.empty() )
+  {
+    MDAL::Log::error( MDAL_Status::Err_FileNotFound, "Mesh file is not valid (null)" );
+    return;
+  }
+
+  auto d = MDAL::DriverManager::instance().driver( driverName );
+
+  if ( !d )
+  {
+    MDAL::Log::error( MDAL_Status::Err_MissingDriver, "No driver with name: " + driverName );
+    return;
+  }
+
+  if ( !d->hasCapability( MDAL::Capability::SaveMesh ) )
+  {
+    MDAL::Log::error( MDAL_Status::Err_MissingDriverCapability, "Driver " + driverName + " does not have SaveMesh capability" );
+    return;
+  }
+
+  if ( d->faceVerticesMaximumCount() < MDAL_M_faceVerticesMaximumCount( mesh ) )
+  {
+    MDAL::Log::error( MDAL_Status::Err_IncompatibleMesh, "Mesh is incompatible with driver " + driverName );
+    return;
+  }
+
+  MDAL::DriverManager::instance().save( static_cast< MDAL::Mesh * >( mesh ), uri );
+}
+
 void MDAL_CloseMesh( MDAL_MeshH mesh )
 {
   if ( mesh )
@@ -339,6 +414,80 @@ void MDAL_M_LoadDatasets( MDAL_MeshH mesh, const char *datasetFile )
   MDAL::DriverManager::instance().loadDatasets( m, datasetFile );
 }
 
+int MDAL_M_metadataCount( MDAL_MeshH mesh )
+{
+  if ( !mesh )
+  {
+    MDAL::Log::error( MDAL_Status::Err_IncompatibleMesh, "Mesh is not valid (null)" );
+    return 0;
+  }
+  MDAL::Mesh *m = static_cast< MDAL::Mesh * >( mesh );
+  int len = static_cast<int>( m->metadata.size() );
+  return len;
+}
+
+const char *MDAL_M_metadataKey( MDAL_MeshH mesh, int index )
+{
+  if ( !mesh )
+  {
+    MDAL::Log::error( MDAL_Status::Err_IncompatibleMesh, "Mesh is not valid (null)" );
+    return EMPTY_STR;
+  }
+  MDAL::Mesh *m = static_cast< MDAL::Mesh * >( mesh );
+  int len = static_cast<int>( m->metadata.size() );
+  if ( len <= index )
+  {
+    MDAL::Log::error( MDAL_Status::Err_IncompatibleMesh, "Requested index: " + std::to_string( index ) + " is out of scope for metadata" );
+    return EMPTY_STR;
+  }
+  size_t i = static_cast<size_t>( index );
+  return _return_str( m->metadata[i].first );
+}
+
+const char *MDAL_M_metadataValue( MDAL_MeshH mesh, int index )
+{
+  if ( !mesh )
+  {
+    MDAL::Log::error( MDAL_Status::Err_IncompatibleMesh,  "Mesh is not valid (null)" );
+    return EMPTY_STR;
+  }
+  MDAL::Mesh *m = static_cast< MDAL::Mesh * >( mesh );
+  int len = static_cast<int>( m->metadata.size() );
+  if ( len <= index )
+  {
+    MDAL::Log::error( MDAL_Status::Err_IncompatibleMesh, "Requested index: " + std::to_string( index ) + " is out of scope for metadata" );
+    return EMPTY_STR;
+  }
+  size_t i = static_cast<size_t>( index );
+  return _return_str( m->metadata[i].second );
+}
+
+void MDAL_M_setMetadata( MDAL_MeshH mesh, const char *key, const char *val )
+{
+  if ( !mesh )
+  {
+    MDAL::Log::error( MDAL_Status::Err_IncompatibleMesh,  "Mesh is not valid (null)" );
+    return;
+  }
+
+  if ( !key )
+  {
+    MDAL::Log::error( MDAL_Status::Err_InvalidData, "Passed pointer key is not valid (null)" );
+    return;
+  }
+
+  if ( !val )
+  {
+    MDAL::Log::error( MDAL_Status::Err_InvalidData, "Passed pointer val is not valid (null)" );
+    return;
+  }
+
+  const std::string k( key );
+  const std::string v( val );
+  MDAL::Mesh *m = static_cast< MDAL::Mesh * >( mesh );
+  m->setMetadata( k, v );
+}
+
 int MDAL_M_datasetGroupCount( MDAL_MeshH mesh )
 {
   if ( !mesh )
@@ -373,6 +522,7 @@ MDAL_DatasetGroupH MDAL_M_datasetGroup( MDAL_MeshH mesh, int index )
     return nullptr;
   }
   size_t i = static_cast<size_t>( index );
+
   return static_cast< MDAL_DatasetH >( m->datasetGroups[i].get() );
 }
 
@@ -790,7 +940,7 @@ MDAL_DatasetH MDAL_G_addDataset( MDAL_DatasetGroupH group, double time, const do
 
   if ( g->dataLocation() == MDAL_DataLocation::DataOnVolumes )
   {
-    MDAL::Log::error( MDAL_Status::Err_MissingDriverCapability, "Dataset Group has data on 3D volumes" );
+    MDAL::Log::error( MDAL_Status::Err_MissingDriverCapability, "Cannot save 3D dataset as a 2D dataset" );
     return nullptr;
   }
 
@@ -813,6 +963,55 @@ MDAL_DatasetH MDAL_G_addDataset( MDAL_DatasetGroupH group, double time, const do
     return nullptr;
 }
 
+MDAL_DatasetH MDAL_G_addDataset3D( MDAL_DatasetGroupH group, double time, const double *values, const int *verticalLevelCount, const double *verticalExtrusions )
+{
+  if ( !group )
+  {
+    MDAL::Log::error( MDAL_Status::Err_IncompatibleDataset, "Dataset Group is not valid (null)" );
+    return nullptr;
+  }
+
+  if ( !values || !verticalLevelCount || !verticalExtrusions )
+  {
+    MDAL::Log::error( MDAL_Status::Err_InvalidData, "Passed pointer Values are not valid" );
+    return nullptr;
+  }
+
+  MDAL::DatasetGroup *g = static_cast< MDAL::DatasetGroup * >( group );
+  if ( !g->isInEditMode() )
+  {
+    MDAL::Log::error( MDAL_Status::Err_IncompatibleDataset, "Dataset Group is not in edit mode" );
+    return nullptr;
+  }
+
+  const std::string driverName = g->driverName();
+  std::shared_ptr<MDAL::Driver> dr = MDAL::DriverManager::instance().driver( driverName );
+  if ( !dr )
+  {
+    MDAL::Log::error( MDAL_Status::Err_MissingDriver, "Driver name " + driverName + " saved in dataset group could not be found" );
+    return nullptr;
+  }
+
+  if ( g->dataLocation() != MDAL_DataLocation::DataOnVolumes )
+  {
+    MDAL::Log::error( MDAL_Status::Err_MissingDriverCapability, "Cannot write 3D data to a Dataset Group that does not have Data On Volumes" );
+    return nullptr;
+  }
+
+  const size_t index = g->datasets.size();
+  MDAL::RelativeTimestamp t( time, MDAL::RelativeTimestamp::hours );
+  dr->createDataset( g,
+                     t,
+                     values,
+                     verticalLevelCount,
+                     verticalExtrusions
+                   );
+  if ( index < g->datasets.size() ) // we have new dataset
+    return static_cast< MDAL_DatasetGroupH >( g->datasets[ index ].get() );
+  else
+    return nullptr;
+}
+
 bool MDAL_G_isInEditMode( MDAL_DatasetGroupH group )
 {
   if ( !group )
@@ -1215,6 +1414,18 @@ const char *MDAL_DR_writeDatasetsSuffix( MDAL_DriverH driver )
   return _return_str( d->writeDatasetOnFileSuffix() );
 }
 
+const char *MDAL_DR_saveMeshSuffix( MDAL_DriverH driver )
+{
+  if ( !driver )
+  {
+    MDAL::Log::error( MDAL_Status::Err_MissingDriver, "Driver is not valid (null)" );
+    return EMPTY_STR;
+  }
+
+  MDAL::Driver *d = static_cast< MDAL::Driver * >( driver );
+  return _return_str( d->saveMeshOnFileSuffix() );
+}
+
 MDAL_MeshH MDAL_CreateMesh( MDAL_DriverH driver )
 {
   if ( !driver )
@@ -1285,3 +1496,28 @@ void MDAL_M_setProjection( MDAL_MeshH mesh, const char *projection )
 
   static_cast<MDAL::Mesh *>( mesh )->setSourceCrsFromWKT( std::string( projection ) );
 }
+
+void MDAL_M_addEdges( MDAL_MeshH mesh,
+                      int edgesCount,
+                      int *startVertexIndices,
+                      int *endVertexIndices )
+{
+  MDAL::Log::resetLastStatus();
+  if ( !mesh )
+  {
+    MDAL::Log::error( MDAL_Status::Err_IncompatibleMesh, "Mesh is not valid (null)" );
+    return;
+  }
+
+  MDAL::Mesh *m = static_cast<MDAL::Mesh *>( mesh );
+
+  if ( ! m->isEditable() )
+  {
+    MDAL::Log::error( MDAL_Status::Err_IncompatibleMesh, "Mesh is not editable" );
+  }
+
+  m->datasetGroups.clear();
+  std::shared_ptr<MDAL::Driver> driver = MDAL::DriverManager::instance().driver( m->driverName() );
+
+  m->addEdges( edgesCount, startVertexIndices, endVertexIndices );
+}
